function prepare_dataset(name)
  % Prepare datasets from SNAP for use in octave/matlab
  % http://snap.stanford.edu/data/index.html
  fprintf('Preparing: %s',name);

  system(sprintf('cat com-%s.ungraph.txt.gz | gunzip | perl -pe ''$_="" if /#/;'' > /tmp/graph',name));
  system(sprintf('cat com-%s.all.cmty.txt.gz | gunzip | perl -pe ''$_="$. $_";s/\t/\n$. /g;'' > /tmp/cmty',name));
  system(sprintf('cat com-%s.top5000.cmty.txt.gz | gunzip | perl -pe ''$_="$. $_";s/\t/\n$. /g;'' > /tmp/cmty5000',name));

  % edges
  g = load('/tmp/graph');
  % relabel the nodes
  if 0
    % unique eats all memory
    [u,v,w] = unique(g(:));
    g = reshape(w,size(g));
    uinv = zeros(1,max(u));
    uinv(u) = 1:length(u);
  else
    delta = 1-min(g(:)); % indices start at 1, not at 0
    if delta ~= 0
      fprintf(' start at %d',1-delta);
    end
    uinv = zeros(1,max(g(:)) + delta);
    uinv(g(:)+delta) = 1;
    uinv(uinv~=0) = 1:nnz(uinv);
    g = uinv(g+delta);
  end

  % graph as sparse matrix
  num_node = max(g(:));
  A = sparse(g(:,1),g(:,2),1,num_node,num_node);
  clear g;
  A = A|A';
  A = double(A);
  
  % clusters
  C = load('/tmp/cmty');
  C(:,2) = uinv(C(:,2)+delta);
  num_clus = max(C(:,1));
  CAll = sparse(C(:,2),C(:,1),1,num_node,num_clus);
  clear C;

  % top 5000 clusters
  C = load('/tmp/cmty5000');
  C(:,2) = uinv(C(:,2)+delta);
  num_clus = max(C(:,1));
  C5k = sparse(C(:,2),C(:,1),1,num_node,num_clus);
  clear C;

  % top 5000 clusters (>=20)
  C5k20 = C5k(:,sum(C5k)>=20);
  
  % save
  C=C5k20;
  if exist('OCTAVE_VERSION','builtin')
    save('-v7', sprintf('com-%s.mat',name),'A','C','C5k','CAll');
  else
    save('-v7.3', sprintf('com-%s.mat',name),'A','C','C5k','CAll');
  end
  %save('-binary',sprintf('com-%s.bin',name),'A','C','C5k','CAll');

  fprintf(',  %d nodes, %d edges, %d clus, %.2f avg.mem, %.2f max mem, %.2f mean size\n', length(A),nnz(A)/2, size(C,2), full(mean(sum(C'))), full(max(sum(C'))), full(mean(sum(C))));
end
