function datasets = datasets_for_experiments(varargin)
  % Get a cell array of all datasets to run experiments on.
  % these can be passed to load_dataset()
  
  % if num_reps>1, then we should use multiple repetitions with different seeds
  % if num_dups>1, then we don't need repetitions, instead multiple copies of the clusters (but the same graph)
  
  opts = struct(varargin{:});
  if ~isfield(opts,'include_big')   opts.include_big   = 1; end
  if ~isfield(opts,'include_lfr')   opts.include_lfr   = 1; end
  if ~isfield(opts,'include_5k')    opts.include_5k    = 1; end
  if ~isfield(opts,'include_5k_combined') opts.include_5k_combined = 0; end
  if ~isfield(opts,'include_5k10')  opts.include_5k10  = opts.include_5k; end
  if ~isfield(opts,'include_5k20')  opts.include_5k20  = 0; end
  if ~isfield(opts,'include_lj')    opts.include_lj    = 1; end
  if ~isfield(opts,'include_orkut') opts.include_orkut = opts.include_big; end
  
  datasets = {};
  
  % LFR benchmark
  
  if opts.include_lfr
    datasets{end+1} = struct('name','lfr-mu0.3-om1', 'type','lfr','mu',0.3,'om',1, 'num_reps',4, 'display_name','LFR (om=1)');
    datasets{end+1} = struct('name','lfr-mu0.3-om2', 'type','lfr','mu',0.3,'om',2, 'num_reps',4, 'display_name','LFR (om=2)');
    datasets{end+1} = struct('name','lfr-mu0.3-om3', 'type','lfr','mu',0.3,'om',3, 'num_reps',4, 'display_name','LFR (om=3)');
    datasets{end+1} = struct('name','lfr-mu0.3-om4', 'type','lfr','mu',0.3,'om',4, 'num_reps',4, 'display_name','LFR (om=4)');
  end
  
  % Small clustering datasets
  
  datasets{end+1} = struct('name','zachary',  'display_name','Karate', 'num_dups',200);
  datasets{end+1} = struct('name','football', 'display_name','Football', 'num_dups',80);
  datasets{end+1} = struct('name','polblogs', 'display_name','Pol.Blogs', 'num_dups',300);
  datasets{end+1} = struct('name','polbooks', 'display_name','Pol.Books', 'num_dups',300);
  
  % More datasets
  
  datasets{end+1} = struct('name','flickr', 'display_name','Flickr', 'num_dups',5);
  
  % SNAP datasets
  
  datasets{end+1} = struct('name','amazon',      'display_name','Amazon','display_variant','All');
  if opts.include_5k
    datasets{end+1} = struct('name','amazon-5k',   'display_name','Amazon','display_variant','Top 5000');
  elseif opts.include_5k_combined
    datasets{end+1} = struct('name','amazon-5k',   'display_name','Amazon/5k');
  end
  if opts.include_5k10
    datasets{end+1} = struct('name','amazon-5k10', 'display_name','Amazon','display_variant','Top 5000, $|C| \ge 10$');
  end
  if opts.include_5k20
    datasets{end+1} = struct('name','amazon-5k20', 'display_name','Amazon','display_variant','Top 5000, $|C| \ge 20$');
  end
  
  datasets{end+1} = struct('name','dblp',                   'display_name','DBLP','display_variant','All');
  if opts.include_5k
    datasets{end+1} = struct('name','dblp-5k',                'display_name','DBLP','display_variant','Top 5000');
  elseif opts.include_5k_combined
    datasets{end+1} = struct('name','dblp-5k',                'display_name','DBLP/5k');
  end
  if opts.include_5k10
    datasets{end+1} = struct('name','dblp-5k10', 'num_reps',1,'display_name','DBLP','display_variant','Top 5000, $|C| \ge 10$');
  end
  if opts.include_5k20
    datasets{end+1} = struct('name','dblp-5k20', 'num_reps',2,'display_name','DBLP','display_variant','Top 5000, $|C| \ge 20$');
  end
  
  datasets{end+1} = struct('name','youtube',     'display_name','Youtube','display_variant','All');
  if opts.include_5k
    datasets{end+1} = struct('name','youtube-5k',  'display_name','Youtube','display_variant','Top 5000');
  elseif opts.include_5k_combined
    datasets{end+1} = struct('name','youtube-5k',  'display_name','Youtube/5k');
  end
  if opts.include_5k10
    datasets{end+1} = struct('name','youtube-5k10','display_name','Youtube','display_variant','Top 5000, $|C| \ge 10$');
  end
  if opts.include_5k20
    datasets{end+1} = struct('name','youtube-5k20','display_name','Youtube','display_variant','Top 5000, $|C| \ge 20$');
  end
  
  if opts.include_lj
    datasets{end+1} = struct('name','lj',      'display_name','LiveJournal','display_variant','All');
    if opts.include_5k
      datasets{end+1} = struct('name','lj-5k',   'display_name','LiveJournal','display_variant','Top 5000');
    elseif opts.include_5k_combined
      datasets{end+1} = struct('name','lj-5k',   'display_name','LiveJournal/5k');
    end
    if opts.include_5k10
      datasets{end+1} = struct('name','lj-5k10', 'display_name','LiveJournal','display_variant','Top 5000, $|C| \ge 10$');
    end
    if opts.include_5k20
      datasets{end+1} = struct('name','lj-5k20', 'display_name','LiveJournal','display_variant','Top 5000, $|C| \ge 20$');
    end
  end
  
  if opts.include_orkut
    datasets{end+1} = struct('name','orkut',      'display_name','Orkut','display_variant','All');
    if opts.include_5k
      datasets{end+1} = struct('name','orkut-5k',   'display_name','Orkut','display_variant','Top 5000');
    elseif opts.include_5k_combined
      datasets{end+1} = struct('name','orkut-5k',   'display_name','Orkut/5k');
    end
    if opts.include_5k10
      datasets{end+1} = struct('name','orkut-5k10', 'display_name','Orkut','display_variant','Top 5000, $|C| \ge 10$');
    end
    if opts.include_5k20
      datasets{end+1} = struct('name','orkut-5k20', 'display_name','Orkut','display_variant','Top 5000, $|C| \ge 20$');
    end
  end
  
  % Yeast data
  
  datasets{end+1} = struct('name','yeast-cyc2008-gavin-2006',    'display_name','CYC/Gavin 2006', 'num_dups',2);
  datasets{end+1} = struct('name','yeast-cyc2008-krogan-2006',   'display_name','CYC/Krogan 2006', 'num_dups',2);
  datasets{end+1} = struct('name','yeast-cyc2008-collins-2007',  'display_name','CYC/Collins 2007', 'num_dups',2);
  datasets{end+1} = struct('name','yeast-cyc2008-costanzo-2010', 'display_name','CYC/Costanzo 2010', 'num_dups',2);
  datasets{end+1} = struct('name','yeast-cyc2008-hoppins-2011',  'display_name','CYC/Hoppins 2011', 'num_dups',2);
  datasets{end+1} = struct('name','yeast-cyc2008',               'display_name','CYC/all', 'num_dups',2);
  
  % Fill in some missing fields
  for id = 1:numel(datasets)
    if ~isfield(datasets{id},'num_reps')
      datasets{id}.num_reps = 1;
    end
    if ~isfield(datasets{id},'num_dups')
      datasets{id}.num_dups = 1;
    end
    if ~isfield(datasets{id},'display_name')
      datasets{id}.display_name = datasets{id}.name;
    end
    if ~isfield(datasets{id},'display_variant')
      datasets{id}.display_variant = '';
    end
  end
end
