
function [A,c] = generate_lfr_graph(params, seed)
  % Call santo fortunato's graph generator
  
  % default parameters
  if nargin < 1, params = ''; end

  windows = any(pwd=='\');
  if windows
    dev_null = 'nul';
  else
    dev_null = '/dev/null';
  end
  if exist('file_in_path') == 5
    exe = file_in_path(path,'benchmark');
    if isempty(exe)
      exe = file_in_path(path,'benchmark.exe');
    end
  else
    exe = which('benchmark');
    exe = exe(1:end-2);
  end
  if isempty(exe)
    if windows
      exe = [pwd '\..\santofortunato\binary_networks\benchmark'];
    else
      exe = [pwd '/../santofortunato/binary_networks/benchmark'];
    end
  end
  
  % work in a temp directory
  cur_pwd = pwd;
  temp_pwd = tempname;
  mkdir(temp_pwd);
  chdir(temp_pwd);
  
  try
    if nargin < 2 || seed == -1
      if ~exist('time')
        time = now;
      end
      seed = int32(rand*1e9) + time;
    end
    dlmwrite(['time_seed.dat'],seed);
  
    default_params = '-N 1000 -k 32 -maxk 128 -minc 10 -mu 0.1';
    cmd = sprintf('%s %s %s > %s',exe,default_params,params,dev_null);
    system(cmd);
  
    net = load('network.dat');
    com = dlmread('community.dat');
    delete('network.dat');
    delete('community.dat');
    delete('statistics.dat');
    delete('time_seed.dat');
    chdir(cur_pwd);
    rmdir(temp_pwd);
    
  catch
    delete('network.dat');
    delete('community.dat');
    delete('statistics.dat');
    delete('time_seed.dat');
    chdir(cur_pwd);
    rmdir(temp_pwd);
    rethrow(lasterror);
  end
  
  A = spconvert([net,ones(size(net,1),1)]);
  c = com(:,2);
  [u,v,w] = find(com(:,2:end));
  c = sparse(u,w,1);
  
  if 0
    % sort for easy visualization
    [c,p] = sort(c);
    A = A(p,p);
  end
end
