function [A,C]=load_dataset(dataset,seed)
  if ~isstruct(dataset)
    dataset = struct('name',dataset, 'type',dataset);
  
    % Snap datasets
    if ~isequal(dataset.type,'snap')
      u = regexp(dataset.name,'^(?<name>dblp|amazon|youtube|dblp|orkut|lj)(-(?<variant>5k|5k20|5k10))?$','names');
      if ~isempty(u.name)
        dataset.type = 'snap';
        dataset.name = u.name;
        dataset.variant = u.variant;
      end
    end
    
    % LFR datasets
    if ~isequal(dataset.type,'lfr')
      u = regexp(dataset.name,'^lfr-mu(?<mu>[0-9.]+)-om(?<om>[0-9]+)$','names');
      if ~isempty(u.mu)
        dataset.type = 'lfr';
        dataset.mu = sscanf(u.mu,'%f');
        dataset.om = sscanf(u.om,'%d');
      end
    end
  end
  if ~isfield(dataset,'type') dataset.type = dataset.name; end
  
  
  % Path
  if exist('file_in_path');
    	data_dir = file_in_path(path,'football.net');
	else
    	data_dir = '';
	end
	if isempty(data_dir)
		basepath = '../data/';
	else
		basepath = data_dir(1:end-12);
	end
	
  % Load a dataset by name
  if isequal(dataset.type,'snap')
    load(sprintf('%scom-%s.mat',basepath,dataset.name));
    if isequal(dataset.variant,'')
      C = CAll(:,sum(CAll) >= 3);
    elseif isequal(dataset.variant,'5k')
      C = C5k(:,sum(C5k) >= 1);
    elseif isequal(dataset.variant,'5k10')
      C = C5k(:,sum(C5k) >= 10);
    elseif isequal(dataset.variant,'5k20')
      C = C5k(:,sum(C5k) >= 20);
    end
    
  elseif isequal(dataset.type,'yeast-cyc2008')
    load([basepath 'ppi-yeast-cyc2008.mat']);
    A = A - diag(diag(A));
  elseif isequal(dataset.type,'yeast-cyc2008-gavin-2006')
    load([basepath 'ppi-yeast-cyc2008-gavin-2006.mat']);
  elseif isequal(dataset.type,'yeast-cyc2008-krogan-2006')
    load([basepath 'ppi-yeast-cyc2008-krogan-2006.mat']);
    A = A - diag(diag(A));
  elseif isequal(dataset.type,'yeast-cyc2008-collins-2007')
    load([basepath 'ppi-yeast-cyc2008-collins-2007.mat']);
  elseif isequal(dataset.type,'yeast-cyc2008-costanzo-2010')
    load([basepath 'ppi-yeast-cyc2008-costanzo-2010.mat']);
  elseif isequal(dataset.type,'yeast-cyc2008-hoppins-2011')
    load([basepath 'ppi-yeast-cyc2008-hoppins-2011.mat']);
    
  elseif isequal(dataset.type,'flickr')
    load([basepath 'flickr.mat']);
		A = A | A';
    
  elseif isequal(dataset.type,'lfr')
    spec = sprintf('-N 5000 -mu %f -k 10 -maxk 50 -t1 2 -t2 1 -minc 20 -maxc 100 -om %d -on 2500', dataset.mu, dataset.om);
    [A,C] = generate_lfr_graph(spec,seed);
    
  elseif isequal(dataset.type,'zachary')
		A = load_pajek([basepath 'zachary.net'], 0);
		A = A | A';
		labels = [1 1 1 1 1 1 1 1 1 2 1 1 1 1 2 2 1 1 2 1 2 1 2 2 2 2 2 2 2 2 2 2 2 2]';
		C = sparse(1:length(A),labels,1);
	elseif isequal(dataset.type,'football')
		A = load_pajek([basepath 'football.net'], 0);
		labels = load([basepath 'football_names.txt']);
		labels = [labels(2:end);labels(1)]; % error in file, 0 became last
		C = sparse(1:length(A),labels+1,1);
	elseif isequal(dataset.type,'polbooks')
		[A,c] = load_pajek([basepath 'polbooks.net'], 0);
		[mapping,junk,labels] = unique(c);
		C = sparse(1:length(A),labels,1);
	elseif isequal(dataset.type,'polblogs')
		[A,c] = load_pajek([basepath 'polblogs.net'], 0);
		labels = cellfun(@str2num,c);
		C = sparse(1:length(A),labels+1,1);
  else
    error('Unknown dataset: %s', dataset.name);
  end
end

