function methods = methods_for_experiments(include_more, include_sigmas, include_auto)
  % A cell array of all the methods to include in the experiments
  % these will be passed as arguments to optimize_cluster
  
  if nargin < 1, include_more = 1; end
  if nargin < 2, include_sigmas = include_more; end
  if nargin < 3, include_auto = 0; end
  
  include_global = 0;
  include_neighbor = include_more;
  include_neighbor_limit = 0;
  include_bfss = 0;
  include_experimental = include_more;
  if include_sigmas
    %sigmas = [0.05:0.05:0.5,0.6:0.1:1];
    %sigmas = [0.01:0.01:0.04,0.05:0.05:1];
    sigmas = [0.05:0.05:2];
  else
    sigmas = [];
  end
  sigmas_with_0 = [0,sigmas];
  em_sigmas = sigmas;
  
  methods = {};
  
  % Gradient methods
  if include_neighbor_limit
    methods{end+1} = struct( ...
      'display_name', 'PGD', ...
      'name', 'gradient-neighbor-limit', ...
      'method', 'gradient', ...
      'subset_method', 'neighbors', ...
      'subset_path_length', 3 ...
    );
    for is=1:numel(sigmas)
      sigma = sigmas(is);
      methods{end+1} = struct( ...
        'display_name', sprintf('$\\sigma=%g$',sigma), ...
        'name', sprintf('gradient-sigma-%g',sigma), ...
        'method', 'gradient', ...
        'score', 'sigma', ...
        'sigma', sigma, ...
        'subset_method', 'neighbors', ...
        'subset_path_length', 3 ...
      );
    end
  end
  if include_bfss
    for is=1:numel(sigmas_with_0)
      sigma = sigmas_with_0(is);
      methods{end+1} = struct( ...
        'display_name', sprintf('$\\sigma=%g$',sigma), ...
        'name', sprintf('gradient-sigma-%g-bfss-3-1000',sigma), ...
        'method', 'gradient', ...
        'score', 'sigma', ...
        'sigma', sigma, ...
        'subset_method', 'bfss', ...
        'subset_path_length', 3, ...
        'subset_size', 1000 ...
      );
    end
  end
  for is=1:numel(sigmas_with_0)
    sigma = sigmas_with_0(is);
    methods{end+1} = struct( ...
      'display_name', sprintf('{\\pgds{%g}}',sigma), ...
      'name', sprintf('gradient-sigma-%g-neighbors-strict-size-3-1000',sigma), ...
      'method', 'gradient', ...
      'score', 'sigma', ...
      'sigma', sigma, ...
      'subset_method', 'neighbors_strict_size', ...
      'subset_size', 1000 ...
    );
  end
  if include_experimental
    %methods{end+1} = struct(
    %  'display_name', 'PGD-bfss-3-1000',
    %  'name', 'gradient-bfss-3-1000',
    %  'method', 'gradient',
    %  'subset_method', 'bfss',
    %  'subset_path_length', 3,
    %  'subset_size', 1000
    %);
    methods{end+1} = struct( ...
      'display_name', 'PGD-no-seed', ...
      'name', 'gradient-no-seed-bfss-3-1000', ...
      'method', 'gradient', ...
      'keep_initial', false, ...
      'subset_method', 'bfss', ...
      'subset_path_length', 3, ...
      'subset_size', 1000 ...
    );
    methods{end+1} = struct( ...
      'display_name', 'PGD-bfs-200', ...
      'name', 'gradient-bfs-2-200', ...
      'method', 'gradient', ...
      'subset_method', 'bfs', ...
      'subset_path_length', 2, ...
      'subset_size', 200 ...
    );
  end
  if include_auto
    if include_experimental
      methods{end+1} = struct( ...
        'display_name', 'PGD-ld', ...
        'name', 'gradient-neighbors-strict-size-automatic-sigma-largest-drop', ...
        'method', 'ad-hoc' ...
      );
      methods{end+1} = struct( ...
        'display_name', 'PGD-$\times2$', ...
        'name', 'gradient-neighbors-strict-size-automatic-sigma-threshold-factor2', ...
        'method', 'ad-hoc' ...
      );
      methods{end+1} = struct( ...
        'display_name', 'PGD-$f\sqrt{n}$', ...
        'name', 'gradient-neighbors-strict-size-automatic-sigma-conductance-sqrt-size', ...
        'method', 'ad-hoc' ...
      );
      methods{end+1} = struct( ...
        'display_name', 'PGD-bs', ...
        'name', 'gradient-neighbors-strict-size-automatic-sigma-big-small', ...
        'method', 'ad-hoc' ...
      );
      methods{end+1} = struct( ...
        'display_name', 'PGD-lc', ...
        'name', 'gradient-neighbors-strict-size-automatic-sigma-last-const', ...
        'method', 'ad-hoc' ...
      );
      methods{end+1} = struct( ...
        'display_name', 'PGD-ld', ...
        'name', 'gradient-neighbors-strict-size-automatic-sigma-density', ...
        'method', 'ad-hoc' ...
      );
    end
    if include_bfss
      methods{end+1} = struct( ...
        'display_name', 'PGD-bfss-d', ...
        'name', 'gradient-automatic-sigma-density', ...
        'method', 'ad-hoc' ...
      );
    end
    if true
      methods{end+1} = struct( ...
        'display_name', '{\pgdd}', ...
        'name', 'gradient-neighbors-strict-size-automatic-sigma-max-density', ...
        'method', 'ad-hoc' ...
      );
    end
  end
  
  % EM/kmeans style methods
  if include_neighbor_limit
    methods{end+1} = struct( ...
      'display_name','EM', ...
      'name','em-sigma-0-lambda-0', ...
      'method','em', ...
      'sigma',0, ...
      'lambda',0, ...
      'subset_method','neighbors', ...
      'subset_path_length',3 ...
    );
    for is=1:numel(em_sigmas)
      sigma = em_sigmas(is);
      lambda = sigma;
      methods{end+1} = struct( ...
        'display_name',sprintf('$EM,\\sigma=%g$',sigma), ...
        'name',sprintf('em-sigma-%g-lambda-%g',sigma,lambda), ...
        'method','em', ...
        'sigma',sigma, ...
        'lambda',lambda, ...
        'subset_method','neighbors', ...
        'subset_path_length',3 ...
      );
    end
  end
  if include_bfss
    for is=1:numel(sigmas_with_0)
      sigma = sigmas_with_0(is);
      lambda = sigma;
      methods{end+1} = struct( ...
        'display_name',sprintf('$EM,\\sigma=%g$,bfss',sigma), ...
        'name',sprintf('em-sigma-%g-lambda-%g-bfss-3-1000',sigma,lambda), ...
        'method','em', ...
        'sigma',sigma, ...
        'lambda',lambda, ...
        'subset_method', 'bfss', ...
        'subset_path_length', 3, ...
        'subset_size', 1000 ...
      );
    end
  end
  if true
    for is=1:numel(sigmas_with_0)
      sigma = sigmas_with_0(is);
      lambda = sigma;
      methods{end+1} = struct( ...
        'display_name',sprintf('{\\emcs{%g}}',sigma), ...
        'name',sprintf('em-sigma-%g-lambda-%g-neighbors-strict-size-1000',sigma,lambda), ...
        'method','em', ...
        'sigma',sigma, ...
        'lambda',lambda, ...
        'subset_method', 'neighbors_strict_size', ...
        'subset_size', 1000 ...
      );
    end
    if include_auto
      methods{end+1} = struct( ...
        'display_name', '{\emcd}', ...
        'name', 'em-neighbors-strict-size-automatic-sigma-max-density', ...
        'method', 'ad-hoc' ...
      );
    end
  end
  
  if include_experimental
    methods{end+1} = struct( ...
      'display_name','greedy', ...
      'name', 'greedy', ...
      'method', 'greedy' ...
    );
    methods{end+1} = struct( ...
      'display_name','greedy${}^-$', ...
      'name', 'greedy_remove', ...
      'method', 'greedy', ...
      'remove', 1 ...
    );
    methods{end+1} = struct( ...
      'display_name','greedy${}_{10}$', ...
      'name', 'greedy_best_10', ...
      'method', 'greedy', ...
      'remove', 1, ...
      'below_best', 0.0, ...
      'max_size', 10 ...
    );
    methods{end+1} = struct( ...
      'display_name','greedy${}_{100}$', ...
      'name', 'greedy_best_100', ...
      'method', 'greedy', ...
      'remove', 1, ...
      'below_best', 0.0, ...
      'max_size', 100 ...
    );
    methods{end+1} = struct( ...
      'display_name','greedy${}_{1000}$', ...
      'name', 'greedy_best_1000', ...
      'method', 'greedy', ...
      'remove', 1, ...
      'below_best', 0.0, ...
      'max_size', 1000 ...
    );
  end
  
  % Comparison methods
  methods{end+1} = struct( ...
    'display_name', '{\YL}', ...
    'name', 'pagerank-nibble', ...
    'method', 'sort', ...
    'subset_method','positive_pagerank', ...
    'rw_decay',0.001, ...
    'rw_iter',100, ...
    'rw_degree_normalize',1, ...
    'optimum_hump',-1.2 ...
  );
  %methods{end+1} = struct( ...
  %  'display_name','LEMON', ...
  %  'name', 'lemon', ...
  %  'method', 'lemon', ...
  %  'skip_data', {{'lj','lj-5k','lj-5k20','lj-5k10','orkut','orkut-5k','orkut-5k20','orkut-5k10'}} ... % Out of memory
  %  %'skip_data', {{'orkut','orkut-5k','orkut-5k20','orkut-5k10'}} ... % Out of memory
  %);
  methods{end+1} = struct( ...
    'display_name','{\HK}', ...
    'name', 'hkgrow', ...
    'method', 'hkgrow' ...
  );
  methods{end+1} = struct( ...
    'display_name','{\PPR}', ...
    'name', 'pprgrow', ...
    'method', 'pprgrow' ...
  );
  if include_experimental
    %methods{end+1} = struct( ...
    %  'display_name','hk(port)', ...
    %  'name', 'heat_kernel', ...
    %  'method', 'heat_kernel' ...
    %);
    %methods{end+1} = struct( ...
    %  'display_name','hk(fixed)', ...
    %  'name', 'hkgrow_fixed', ...
    %  'method', 'hkgrow_fixed' ...
    %);
    methods{end+1} = struct( ...
      'display_name','hk(seed)', ...
      'name', 'hkgrow_union_seed', ...
      'method', 'hkgrow_union_seed' ...
    );
    methods{end+1} = struct( ...
      'display_name','hk(neig)', ...
      'name', 'hkgrow_neighbors', ...
      'method', 'hkgrow_neighbors' ...
    );
  end
  
  % Global methods
  if include_global
    methods{end+1} = struct( ...
      'display_name', 'Louvain', ...
      'name', 'lso-modularity', ...
      'method', 'lso', ...
      'lso_arguments', {{'loss','modularity'}} ...
    );
    methods{end+1} = struct( ...
      'display_name', 'Louvain on $d \le 3$', ...
      'name', 'lso-modularity-neighbors3', ...
      'method', 'lso', ...
      'lso_arguments', {{'loss','modularity'}}, ...
      'subset_method', 'neighbors', ...
      'subset_path_length', 3 ...
    );
    methods{end+1} = struct( ...
      'display_name', 'SLD', ...
      'name', 'lso-sld', ...
      'method', 'lso', ...
      'lso_arguments', {{'loss','sld'}} ...
    );
    methods{end+1} = struct( ...
      'display_name', 'SLD on $d \le 3$', ...
      'name', 'lso-sld-neighbors3', ...
      'method', 'lso', ...
      'lso_arguments', {{'loss','sld'}}, ...
      'subset_method', 'neighbors', ...
      'subset_path_length', 3 ...
    );
  end
  
  % k-neighbors
  if include_neighbor
    methods{end+1} = struct('display_name','$d \le 1$', 'name','1-neighbors', 'method','neighbors', 'path_length',1);
    methods{end+1} = struct('display_name','$d \le 2$', 'name','2-neighbors', 'method','neighbors', 'path_length',2);
    methods{end+1} = struct('display_name','$d \le 3$', 'name','3-neighbors', 'method','neighbors', 'path_length',3);
  end
  if include_experimental
    methods{end+1} = struct( ...
      'display_name', 'all-bfss-3-1000', ...
      'name', 'all-bfss-3-1000', ...
      'method', 'all', ...
      'subset_method', 'bfss', ...
      'subset_path_length', 3, ...
      'subset_size', 1000 ...
    );
  end
end
