function [c,l] = optimize_cluster_addrem(A,p,c0,opts)
  % Loop over nodes, and consider if adding/removing them improves the score
  if ~isfield(opts,'allow_remove'), opts.allow_remove = 1; end
  
  if isequal(opts.order,'pagerank')
    [pi,~,pv] = find(p);
    [~,order] = sort(pv,'descend');
    order = pi(order);
  end

  d = full(sum(A,2));
  if isfield(opts,'sigma')
    A = A + opts.sigma * diag(d); % only for numerator
  end
  c = c0>0;
  w = full(A*c);
  sv = sum(d(c));
  sw = sum(w(c));
  for oit = 1:opts.max_iter
    if opts.verbose >= 1
      fprintf('Iteration %d, nnz %d, sw %f(=%f), sv %f(=%f), s %f\n',oit,nnz(c),sw,c'*A*c,sv,c'*d,sw/(sv+eps));
    end
    if isequal(opts.order,'within')
      s = (w>0) .* (w + rand(size(w))*0.001);
      [pi,~,pv] = find(s);
      [~,order] = sort(pv,'descend');
      order = pi(order);
    elseif isequal(opts.order,'within-asc')
      s = (w>0) .* (w + rand(size(w))*0.001);
      [pi,~,pv] = find(s);
      [~,order] = sort(pv,'ascend');
      order = pi(order);
    elseif isequal(opts.order,'after')
      s = (w>0) .* (2*w+diag(A)+sw + rand(size(w))*0.001) ./ (sv + d);
      [pi,~,pv] = find(s);
      [~,order] = sort(pv,'descend');
      order = pi(order);
    elseif isequal(opts.order,'random')
      order = find(w > 0);
      order = order(randperm(length(order)));
    elseif ~isequal(opts.order,'pagerank')
      error('Unknown order: %s',opts.order);
    end
    
    change = 0;
    for it = 1:length(order);
      i = order(it);
      if ~c(i)
        %if w(i)/v(i) >= sw/sv
        %if (sw+2*w(i)+A(i,i))/(sv+d(i)+eps) >= sw/(sv+eps)
        if (2*w(i)+A(i,i))*sv >= sw*d(i)
          % add node
          c(i) = 1;
          w = w + A(:,i);
          sv = sv + d(i);
          sw = sw + (2*w(i)+A(i,i));
          change = change + 1;
          if opts.verbose >= 2
            fprintf(' added %d, nnz %d, sw %f(=%f), sv %f(=%f), s %f\n',i,nnz(c),sw,c'*A*c,sv,c'*d,sw/(sv+eps));
          end
        end
      elseif opts.allow_remove
        if (2*w(i)+A(i,i))*sv < sw*d(i)
          change = change + 1;
          sw = sw - (2*w(i)+A(i,i));
          sv = sv - d(i);
          w = w - A(:,i);
          c(i) = 0;
          if opts.verbose >= 2
            fprintf(' removed %d, nnz %d, sw %f(=%f), sv %f(=%f), s %f\n',i,nnz(c),sw,c'*A*c,sv,c'*d,sw/(sv+eps));
          end
        end
      end
    end
    if ~change
      break
    end
  end
  l = sw / (sv+eps);
end
