function run_automatic_sigma()
  more off;
  seeds = seeds_for_experiments();
  datasets = datasets_for_experiments();
  %base_methods = {'gradient','em', 'gradient-neighbors-strict-size', 'em-neighbors-strict-size'};
  base_methods = {'gradient-neighbors-strict-size', 'em-neighbors-strict-size'};
  %base_methods = {'gradient-neighbor-limit','em-neighbor-limit'};
  %pick_methods = {'threshold-factor2','largest-drop','conductance-sqrt-size','last-const','big-small','density','max-density'};
  pick_methods = {'density','max-density'};
  
  for is = 1:numel(seeds)
    seed = seeds{is};
    for im = 1:numel(base_methods)
      base_method = base_methods{im};
      for ip = 1:numel(pick_methods)
        pick_method = pick_methods{ip};
        for id = 1:numel(datasets);
          dataset = datasets{id};
          for rep = 1:dataset.num_reps
            run_automatic_sigma_on(base_method,pick_method,dataset,seed,rep);
          end
        end
      end
    end
  end
end

function run_automatic_sigma_on(base_method,pick_method,dataset,seed,rep)
  %sigmas = [0:0.05:0.5,0.6:0.1:1];
  sigmas = [0:0.05:1];
  
  method_name = sprintf('%s-automatic-sigma-%s', base_method, pick_method);
  filename = output_file_for_experiment(method_name,dataset,seed,rep);
  if exist(filename,'file')
    fprintf('[%s] Skipping %s\n',datestr(now,'HH:MM:SS'),filename);
    return;
  else
    fprintf('[%s] Running %s\n',datestr(now,'HH:MM:SS'),filename);
  end
  
  if isequal(base_method,'gradient')
    methods = @(sigma) sprintf('gradient-sigma-%g-bfss-3-1000',sigma);
  elseif isequal(base_method,'em')
    methods = @(sigma) sprintf('em-sigma-%g-lambda-%g-bfss-3-1000',sigma,sigma);
  elseif isequal(base_method,'gradient-neighbor-limit')
    methods = @(sigma) ifelse(sigma==0, 'gradient-neighbor-limit', sprintf('gradient-sigma-%g',sigma));
  elseif isequal(base_method,'em-neighbor-limit')
    methods = @(sigma) sprintf('em-sigma-%g-lambda-%g',sigma,sigma);
  elseif isequal(base_method,'gradient-neighbors-strict-size')
    methods = @(sigma) sprintf('gradient-sigma-%g-neighbors-strict-size-3-1000',sigma);
  elseif isequal(base_method,'em-neighbors-strict-size')
    methods = @(sigma) sprintf('em-sigma-%g-lambda-%g-neighbors-strict-size-1000',sigma,sigma);
  else
    error('Unknown base method: %s', base_method);
  end
  
  outs = cell(numel(sigmas),1);
  for is = 1:numel(sigmas)
    sigma = sigmas(is);
    filename_sigma = output_file_for_experiment(methods(sigma),dataset,seed,rep);
    if ~exist(filename_sigma,'file')
      fprintf('[%s]  Missing %s!\n',datestr(now,'HH:MM:SS'),filename_sigma);
      return;
    end
    load(filename_sigma,'out');
    outs{is} = out;
    clear out;
  end
  
  % first: check that the seeds are the same
  seedvols = cell2mat(cellfun(@(out)out.init.volume,outs,'UniformOutput',false));
  truevols = cell2mat(cellfun(@(out)out.truth.volume,outs,'UniformOutput',false));
  assert(all(seedvols == repmat(seedvols(1,:),length(sigmas),1)));
  assert(all(truevols == repmat(truevols(1,:),length(sigmas),1)));
  
  % difference in log size
  nnzs = cell2mat(cellfun(@(out)out.found.nnz,outs,'UniformOutput',false));
  ws   = cell2mat(cellfun(@(out)out.found.within_th,outs,'UniformOutput',false));
  vs   = cell2mat(cellfun(@(out)out.found.volume_th,outs,'UniformOutput',false));
  assert(isequal(size(nnzs), [length(sigmas),length(outs{1}.found.nnz)]))
  dlog = diff(log(nnzs));
  
  cputimes = cell2mat(cellfun(@(out)out.cputime,outs,'UniformOutput',false));
  used = ones(size(cputimes));
  
  if isequal(pick_method,'largest-drop')
    % pick the sigma after the largest drop in log cluster size
    [~,jump] = min(dlog);
    pick = jump+1;
  elseif isequal(pick_method,'threshold-0.3')
    [~,pick] = min(dlog >= -0.3);
  elseif isequal(pick_method,'threshold-factor2')
    [~,pick] = min(dlog >= -log(2));
  elseif isequal(pick_method,'conductance-sqrt-size')
    [~,pick] = max(ws ./ (vs .* sqrt(nnzs) + eps));
  elseif isequal(pick_method,'last-const')
    pick = find_last_row(diff(nnzs) == 0 & nnzs(2:end,:) > 1, 1);
    used = bsxfun(@ge, (1:length(sigmas))', pick);
  elseif isequal(pick_method,'density')
    pick = find_last_row(diff(ws./nnzs.^2)>0, 0)+1;
    used = bsxfun(@ge, (1:length(sigmas))', pick-1);
  elseif isequal(pick_method,'max-density')
    [~,pick] = max(ws./nnzs.^2);
  elseif isequal(pick_method,'big-small')
    pick = ifelse((ws ./ vs >= 0.5)(1,:), 1, length(outs));
  else
    error('unknown method: %s', pick_method);
  end 
  
  % pick the values from 'out'
  out = cell(1,length(pick));
  for i=1:length(pick);
    out{i} = index_transposed_struct(outs{pick(i)}, i);
  end
  out = transpose_struct(out);
  out.sigma = sigmas(pick);
  out.method.name = method_name;
  % sum cputimes
  out.cputime = sum(used .* cputimes);

  % save
  save('-v7',filename,'out');
end

function i = find_last_row(xs,def)
  [u,i] = max(flipud(xs));
  i = size(xs,1)+1-i;
  i(~u) = def;
end
