function run_calculate_min_sigma(varargin)
  % For each dataset, calculate the minimal value of sigma to make the clusters local optima of (modified) conductance
  more off;
  
  opts = struct(varargin{:});
  if !isfield(opts,'parallel'), opts.parallel = 3; end
  
  max_cluster = 5000; % consider at most this many clusters
  extra_reps  = 1;
  extra_reps_small = 4;
  datasets = datasets_for_experiments();
  
  for id = 1:numel(datasets)
    dataset = datasets{id};
    % save
    filename = output_file_for_min_sigma_experiment(dataset);
    if exist(filename)
      fprintf('[%s] Skipping %s\n',datestr(now,'HH:MM:SS'),filename);
      continue;
    else
      fprintf('[%s] Running %s\n',datestr(now,'HH:MM:SS'),filename);
    end
    % Repetitions
    reps = dataset.num_reps * dataset.num_dups * extra_reps;
    if reps > 1
      reps = reps * extra_reps_small;
    end
    min_sigmas        = zeros(0,1);
    min_sigmas_random = zeros(0,1);
    sizes             = zeros(0,1);
    for rep = 1:reps
      fprintf('[%s] %d / %d\r',datestr(now,'HH:MM:SS'),rep,reps);
      % Load data
      [A,C] = load_dataset(dataset,10*rep);
      d = full(sum(A));
      % sample some clusters
      which = randperm(size(C,2),min(size(C,2),max_cluster));
      sizes(end+1:end+length(which)) = sum(C(:,which));
      % calculate min sigmas
      min_sigmas(end+1:end+length(which)) = pararrayfun(opts.parallel,@(i)min_sigma(A,C(:,i),d),which);
      % calculate min sigmas for random clusters of the same size
      srand(10*rep+1);
      CR = pararrayfun(opts.parallel,@(i)random_cluster(A,nnz(C(:,i))),which);
      min_sigmas_random(end+1:end+length(which)) = pararrayfun(opts.parallel,@(i)min_sigma(A,CR(:,i),d),1:length(which));
    end
    % save
    save('-v7',filename,'min_sigmas','min_sigmas_random','sizes');
    clear A C d min_sigmas min_sigmas_random
  end
end

