function [r,dr] = score_conductance_close(A,c,c0,w0,d)
  % [r,dr] = score_conductance_close(A,c,c0,w0,d)
  %
  % Input:
  %   A = adjacency matrix
  %   c = cluster membership degrees (sparse column vector)
  %   d = (optional) node degrees, d = sum(A)
  % Output:
  %   r = w/v - w0/2*(c-c0)^2:  the within cluster fraction
  %   dr = ∂s/∂c
  %
  % Calculate the within fraction of a cluster (higher is better).
  % Note: conductance = 1-s
  % Normalizes cluster memberships between 0 and 1, or equivalently calculates.
  % Second output is the gradient.
  
  if nargin<3 || isempty(d)
    d = full(sum(A));
  end
  
  % significantly faster to do the sparse multiplication manually
  [i,j,k] = find(c);
  if length(k)<10000
    w = full(diag( c'*(A(:,i)*k) ))';
  else
    w = full(diag( k'*(A(i,i)*k) ))';
  end
  v = full(d*c);
  r = w./(v+eps) - 0.5*w0*full(sumsq(c-c0));
  
  if nargout>=2
    n = length(A);
    if 0
      dw = 2*A*c;
    else
      dw = 2*A(:,i)*k; % faster multiplication
    end
    dv = d';
    dr = dw/v - dv*(w/v^2) - w0*(c-c0);
    
    dr = sparse(dr);
    dr = dr .* (dr>0 | c~=0); % don't remove nodes that aren't there
    dr(find(c==max(c) & dr>0)) = 0; % don't increase membership that is already 1
  end
end
