function [r,dr] = score_conductance_degree_hack(A,c,n,d)
  % [r,dr] = score_conductance_degree_hack(A,c,n,d)
  %
  % Input:
  %   A = adjacency matrix
  %   c = cluster membership degrees (sparse column vector)
  %   d = (optional) node degrees, d = sum(A)
  % Output:
  %   s = w / v
  %   ds = ∂s/∂c, with d += n
  
  if nargin<4 || isempty(d)
    d = full(sum(A));
  end
  d += n;
  
  [i,j,k] = find(c);
  v = full(d*c);
  w = full(diag( c'*(A(:,i)*k) ))';
  
  r = w ./ (v+eps);
  
  if nargout>=2
    n = length(A);
    dw = 2*A*c;
    dv = d';
    dr = dw./(v+eps) - dv*w/(v+eps).^2;
    
    dr = sparse(dr);
    dr = dr .* (dr>0 | c~=0); % don't remove nodes that aren't there
    dr(find(c==1 & dr>0)) = 0; % don't increase membership that is already 1
  end
end
