function [r,dr] = score_conductance_double(A,c,d)
  % [r,dr] = score_conductance_double(A,c,d)
  %
  % Input:
  %   A = adjacency matrix
  %   c = cluster membership degrees (sparse column vector)
  %   d = (optional) node degrees, d = sum(A)
  % Output:
  %   s = conductance(c) + conductance(1-c)
  %   ds = ∂s/∂c
  
  if nargin<3 || isempty(d)
    d = full(sum(A));
  end
  
  c1 = min(1,max(0,c));
  %c2 = 1-c;
  
  % Assume that c1 is smaller
  % The edges are
  % 
  %          c1   (1-c1)  total
  %  c1      w1   b1      v1
  %  (1-c1)  b1   w2
  %  total   v1           v
  % hence
  %  v2 = v-v1;
  %  w2 = v2-b1 = v2-(v1-w1)
  
  [i1,j1,k1] = find(c1);
  %[i2,j2,k2] = find(c2);
  
  v = full(sum(d));
  v1 = full(d*c1);
  %v2 = full(d*c2);
  v2 = v - v1;
  
  %w1 = full(diag( c1'*(A*c1) ))';
  %w2 = full(diag( c2'*(A*c2) ))';
  w1 = full(diag( c1'*(A(:,i1)*k1) ))';
  %w2 = full(diag( c2'*(A(:,i2)*k2) ))';
  w2 = v2-(v1-w1);
  
  r = w1./(v1+eps) + w2./(v2+eps);
  
  if nargout>=2
    n = length(A);
    dw1 = 2*A*c1;
    %dw2 = 2*A*c2;
    dw2 = 2*d' - dw1;
    dv = d';
    dr = (dw1/(v1+eps) - dw2/(v2+eps)) - dv*(w1/(v1^2+eps) - w2/(v2^2+eps));
    
    dr = sparse(dr);
    dr = dr .* (dr>0 | c~=0); % don't remove nodes that aren't there
    dr(find(c==1 & dr>0)) = 0; % don't increase membership that is already 1
  end
end
