function [r,dr] = score_conductance_plus(A,c,a,b,d)
  % [r,dr] = score_conductance_plus(A,c,a,b,d)
  %
  % Input:
  %   A = adjacency matrix
  %   c = cluster membership degrees (sparse column vector)
  %   d = (optional) node degrees, d = sum(A)
  % Output:
  %   s = (w+a) / (v+b)
  %   ds = ∂s/∂c
  
  if nargin<5 || isempty(d)
    d = full(sum(A));
  end
  
  [i,j,k] = find(c);
  v = full(d*c);
  w = full(diag( c'*(A(:,i)*k) ))';
  
  r = (w+a) ./ (v+b);
  
  if nargout>=2
    n = length(A);
    dw = 2*A*c;
    dv = d';
    dr = dw./(v+b) - dv*(w+a)/(v+b).^2;
    
    dr = sparse(dr);
    dr = dr .* (dr>0 | c~=0); % don't remove nodes that aren't there
    dr(find(c==1 & dr>0)) = 0; % don't increase membership that is already 1
  end
end
