function [r,dr] = score_conductance_plus_rw(A,c,p,l,d)
  % [r,dr] = score_conductance_plus_rw(A,c,p,l,d)
  %
  % Input:
  %   A = adjacency matrix
  %   c = cluster membership degrees (sparse column vector)
  %   p = random walk hitting probabilities
  %   l = relative weight of rw term
  %   d = (optional) node degrees, d = sum(A)
  % Output:
  %   s = (1-l) * cAc / 1Ac + l * cp / 1c
  %   ds = ∂s/∂c
  
  if nargin<5 || isempty(d)
    d = full(sum(A));
  end
  
  [i,j,k] = find(c);
  v = full(d*c);
  w = full(diag( c'*(A(:,i)*k) ))';
  cp = full(c'*p);
  sc = full(sum(c));
  
  r = (1-l) * w ./ (v+eps) + l * cp / (sc + eps);
  
  if nargout>=2
    n = length(A);
    dw = 2*A*c;
    dv = d';
    dr = (1-l) * (dw./(v+eps) - dv*w/(v+eps).^2) + l * (p / (sc+eps) - 1 * cp / (sc+eps));
    
    dr = sparse(dr);
    dr = dr .* (dr>0 | c~=0); % don't remove nodes that aren't there
    dr(find(c==1 & dr>0)) = 0; % don't increase membership that is already 1
  end
end
