function [r,dr] = score_conductance_pow(A,c,power,d)
  % [r,dr] = score_conductance_pow(A,c,power,d)
  %
  % Input:
  %   A = adjacency matrix
  %   c = cluster membership degrees (sparse column vector)
  %   d = (optional) node degrees, d = sum(A)
  % Output:
  %   s = w^power / v
  %   ds = ∂s/∂c
  
  if nargin<3 || isempty(d)
    d = full(sum(A));
  end
  
  [i,j,k] = find(c);
  v = full(d*c);
  w = full(diag( c'*(A(:,i)*k) ))';
  
  r = w.^power ./ (v+eps);
  
  if nargout>=2
    n = length(A);
    dw = 2*A*c;
    dv = d';
    dr = power.*dw.*w.^(power-1)/(v+eps) - dv*w.^power/(v.^2+eps);
    
    dr = sparse(dr);
    dr = dr .* (dr>0 | c~=0); % don't remove nodes that aren't there
    dr(find(c==1 & dr>0)) = 0; % don't increase membership that is already 1
  end
end
