function [r,dr] = score_conductance_sigma_lambda(A,c,d, sigma,lambda1,lambda2)
  % [r,dr] = score_conductance_sigma_lambda(A,c,d, sigma,lambda1,lambda2)
  %
  % Input:
  %   A = adjacency matrix
  %   c = cluster membership degrees (sparse column vector)
  %   d = (optional) node degrees, d = sum(A)
  % Output:
  %   s = -∑c_i d_i |x_i-μ_i|^2 - ∑(1-c_i)(λ₁ + d_i λ₂)
  %     = (w + ∑d_ic_i^2*sigma)/v - ∑c_i*(a_ii/d_i+sigma-lambda1-lambda2*d_i)
  %   ds = ∂s/∂c
  %   assumes that a_ii = 0.
  
  [i,j,k] = find(c);
  v = full(d*c);
  w = full(diag( c'*(A(:,i)*k) ))';
  
  n = sum(c);
  v2 = full(d*(c.^2));
  %d  = sum(diag(A) .* c ./ (d+eps));
  r = (w + v2*sigma) ./ (v+eps) - n*sigma + lambda1*n + lambda2*v;
  
  if nargout>=2
    n = length(A);
    dw = 2*A*c + 2*sigma*c.*d';
    dv = d';
    dr = dw./(v+eps) - dv*(w + v2*sigma)/(v+eps).^2 - sigma + lambda1 + lambda2*dv;
    
    dr = sparse(dr);
    dr = dr .* (dr>0 | c~=0); % don't remove nodes that aren't there
    dr(find(c==1 & dr>0)) = 0; % don't increase membership that is already 1
  end
end
