function [r,dr] = score_conductance_times_local(A,c,d)
  % [r,dr] = score_conductance_times_local(A,c,d)
  %
  % Input:
  %   A = adjacency matrix
  %   c = cluster membership degrees (sparse column vector)
  %   d = (optional) node degrees, d = sum(A)
  % Output:
  %   s = w / v * w/(n*(n-1)/2)
  %   ds = ∂s/∂c
  
  if nargin<3 || isempty(d)
    d = full(sum(A));
  end
  
  [i,j,k] = find(c);
  v = full(d*c);
  w = full(diag( c'*(A(:,i)*k) ))';
  n = full(sum(c));
  
  r = w.^2 ./ (v .* n.*(n-1)/2 + eps);
  
  
  if nargout>=2
    n = length(A);
    dw = 2*A*c;
    dv = d';
    dr = 2.*dw.*w./(v .* n.*(n-1)/2 + eps) - (dv .* n.*(n-1)/2 + v .* 2.*(n-1)/2)*w.^2./(v .* n.*(n-1)/2 + eps).^2;
    
    dr = sparse(dr);
    dr = dr .* (dr>0 | c~=0); % don't remove nodes that aren't there
    dr(find(c==1 & dr>0)) = 0; % don't increase membership that is already 1
  end
end
