function [r,dr] = score_conductance_weighted(A,c,d)
  % [r,dr] = score_conductance_weighted(A,c,d)
  %
  % Input:
  %   A = adjacency matrix
  %   c = cluster membership degrees (sparse column vector)
  %   d = (optional) node degrees, d = sum(A)
  % Output:
  %   s = (∑_ij c_i c_j a_ij/d_i ) / (∑_i ci)
  %   ds = ∂s/∂c
  
  if nargin<3 || isempty(d)
    d = full(sum(A));
  end
  d = d';
  
  [i,j,k] = find(c);
  w = full(diag( (c./d)'*(A(:,i)*k) ))';
  n = sum(c)+eps;
  
  r = w ./ n;
  
  if nargout>=2
    n = length(A);
    dw = A*(c./d) + (A*c)./d;
    dn = 1;
    dr = dw./n - dn*w/n.^2;
    
    dr = sparse(dr);
    dr = dr .* (dr>0 | c~=0); % don't remove nodes that aren't there
    dr(find(c==1 & dr>0)) = 0; % don't increase membership that is already 1
  end
end

