function write_min_sigma_plots()
  range = 0:0.02:2;
  outdir = '../tables';
  datasets = datasets_for_experiments();
  
  hs = zeros(numel(datasets),length(range));
  for id = 1:numel(datasets)
    % load
    dataset = datasets{id};
    filename = output_file_for_min_sigma_experiment(dataset);
    if ~exist(filename)
      warning("No min sigma results for %s",dataset.name);
      continue;
    end
    load(filename,'min_sigmas','min_sigmas_random');
    
    % write and collect
    h  = collect_min_sigmas(range, min_sigmas);
    hr = collect_min_sigmas(range, min_sigmas_random);
    write_min_sigma_plot(sprintf('%s/plot-min-sigma-%s.tex',       outdir,dataset.name), range, h);
    write_min_sigma_plot(sprintf('%s/plot-min-sigma-random-%s.tex',outdir,dataset.name), range, hr);
    write_min_sigma_plot(sprintf('%s/plot-min-sigma-diff-%s.tex',  outdir,dataset.name), range, h-hr);
    write_min_sigma_plot(sprintf('%s/plot-min-sigma-ratio-%s.tex',  outdir,dataset.name), range, h./(h+hr+eps));
    hs(id,:) = h;
    
    clear min_sigmas min_sigmas_random;
  end
  
  % plot
  plot(range,hs);
end

function h = collect_min_sigmas(range, min_sigmas)
  h = mean(bsxfun(@lt,min_sigmas',range));
end

function write_min_sigma_plot(filename, range, h)
  file = fopen(filename,'wt');
  fprintf(file,'\\addplot+[] coordinates{');
  for i=1:numel(range)
    fprintf(file,'(%f,%f) ',range(i),h(i));
  end
  fprintf(file,'};\n');
  fclose(file);
end

