function write_results_tables(varargin)
  opts = struct(varargin{:});
  if ~isfield(opts,'outdir'), opts.outdir = '../tables'; end
  if ~isfield(opts,'include_plots'), opts.include_plots = 0; end
  if ~isfield(opts,'include_variants'), opts.include_variants = 0; end
  more off;
  
  seeds = seeds_for_experiments();
  datasets = datasets_for_experiments();
  
  variants = {};
  variants{end+1} = struct( ...
    'name', '', ...
    'variants', {{'','All'}}, ...
    'variants_column', 0, ...
    'more_methods', 0, ...
    'include_sigmas', 0, ...
    'include_auto', 1 ...
  );
  if opts.include_variants
  variants{end+1} = struct( ...
    'name', '-all', ...
    'variants', {{}}, ...
    'variants_column', 1, ...
    'more_methods', 1, ...
    'include_sigmas', 0, ...
    'include_auto', 1 ...
  );
  variants{end+1} = struct( ...
    'name', '-5k', ...
    'variants', {{'','All','Top 5000','Top 5000, $|C| \ge 10$'}}, ...
    'variants_column', 1, ...
    'more_methods', 0, ...
    'include_sigmas',0, ...
    'include_auto', 1 ...
  );
  variants{end+1} = struct( ...
    'name', '-5k10', ...
    'variants', {{'Top 5000, $|C| \ge 10$'}}, ...
    'variants_column', 1, ...
    'more_methods', 1, ...
    'include_sigmas',0, ...
    'include_auto', 1 ...
  );
  end
  
  %sigmas = [0:0.05:0.5,0.6:0.1:1];
  sigmas = [0:0.05:2];
  sigma_methods = {};
  %pgd_methods = @(sigma) ifelse(sigma==0, 'gradient-neighbor-limit', sprintf('gradient-sigma-%g',sigma));
  %em_methods  = @(sigma) sprintf('em-sigma-%g-lambda-%g',sigma,sigma);
  %sigma_methods{end+1} = struct('name', 'pgd',     'methods',@(sigma) sprintf('gradient-sigma-%g-bfss-3-1000',sigma));
  sigma_methods{end+1} = struct('name', 'pgd', 'methods',@(sigma) sprintf('gradient-sigma-%g-neighbors-strict-size-3-1000',sigma));
  %sigma_methods{end+1} = struct('name', 'em',      'methods',@(sigma) sprintf('em-sigma-%g-lambda-%g-bfss-3-1000',sigma,sigma));
  sigma_methods{end+1} = struct('name', 'em',  'methods',@(sigma) sprintf('em-sigma-%g-lambda-%g-neighbors-strict-size-1000',sigma,sigma));
  
  quantities = {'f1-scores','f1-scores-th','f1-no-seed-scores','cluster-size','cluster-size-th','conductance','conductance-th','cputime'};
  for iq = 1:numel(quantities)
    quantity = quantities{iq};
    for is = 1:numel(seeds)
      seed = seeds{is};
      for iv = 1:numel(variants)
        variant = variants{iv};
        filename = sprintf('%s/table-%s-%s-seeds%s.tex',opts.outdir,quantity,seed.name,variant.name);
        write_results_table(filename, quantity, seed, variant);
      end
      
      if opts.include_plots
        for id = 1:numel(datasets)
          dataset = datasets{id};
          for ism = 1:numel(sigma_methods)
            sigma_method = sigma_methods{ism};
            filename = sprintf('%s/plot-%s-sigma-%s-%s-seeds-on-%s.tex', opts.outdir, sigma_method.name, quantity, seed.name, dataset.name);
            write_results_plot(filename, quantity, dataset, seed, sigmas, sigma_method.methods);
          end
        end
      end
    end
  end
end

function [xs,xs_true,datasets,methods] = collect_results(quantity, seed, variant)
  % Collect statistics
  datasets = datasets_for_experiments();
  methods  = methods_for_experiments(variant.more_methods, variant.include_sigmas, variant.include_auto);
  
  if nargin >= 3 && ~isempty(variant.variants)
    keep = @(d) (isfield(d,'display_variant') && ismember(d.display_variant,variant.variants));
    datasets = datasets(cellfun(keep,datasets));
  end
  
  xs       = cell(numel(datasets), numel(methods));
  xs_true  = cell(numel(datasets));
  
  for id = 1:numel(datasets)
    dataset = datasets{id};
    for im = 1:numel(methods)
      method = methods{im};
      [xs_im,xs_im_true] = stats_for_experiment(quantity, method, dataset, seed);
      xs{id,im}   = xs_im;
      xs_true{id} = [xs_true{id}, xs_im_true];
    end
  end
  
  % Only include the datasets/methods for which we have any output
  good = ~cellfun(@isempty,xs);
  methods   = methods (any(good,1));
  datasets  = datasets(any(good,2));
  xs        = xs(any(good,2),any(good,1));
  xs_true   = xs_true(any(good,2));
end

function [xs,xs_true] = stats_for_experiment(quantity, method, dataset, seed);
  xs = [];
  xs_true = [];
  for rep = 1:dataset.num_reps
    [xs_rep,xs_rep_true] = stats_for_experiment_rep(quantity, method, dataset, seed, rep);
    if rep==1 && isempty(xs_rep)
      warning('We have no data for %s',output_file_for_experiment(method,dataset,seed,rep));
    end
    xs = [xs, xs_rep];
    xs_true = [xs_true, xs_rep_true];
  end
end

function [xs_rep,xs_rep_true] = stats_for_experiment_rep(quantity, method, dataset, seed, rep)
  filename = output_file_for_experiment(method,dataset,seed,rep);
  if ~exist(filename,'file')
    if rep > 1
      warning('We have no data for %s, but earlier repetitions exist.',filename);
    end
    xs_rep = [];
    xs_rep_true = [];
    return;
  end
  
  % load output from experiment
  load(filename, 'out');
  
  xs_rep_true = [];
  if isequal(quantity,'f1-scores')
    xs_rep = out.found_vs_truth.f1;
  elseif isequal(quantity,'f1-scores-th')
    if isfield(out.found_vs_truth,'nnz_th_min') 
      xs_rep = 2*out.found_vs_truth.nnz_th_min ./ (out.found.nnz_th + out.truth.nnz_th);
    else
      if ~isequal(method.name,'lemon') % we know this method is discrete, so that's okay.
        warning('No thresholded stats for %s',filename);
      end
      xs_rep = out.found_vs_truth.f1;
    end
  elseif isequal(quantity,'f1-no-seed-scores')
    if ~isfield(out.truth,'sum')
      % reconstruct sum, since earlier versions didn't save it
      if ~isequal(method.name,'lemon')
        warning('No sum stats for %s',filename);
      end
      out.truth.sum = out.truth.nnz;
      out.seed.sum  = out.seed.nnz;
      out.found.sum = 2*out.found_vs_truth.sum_min ./ out.found_vs_truth.f1 - out.truth.nnz;
      bad = out.found_vs_truth.f1<1e-10;
      out.found.sum(bad) = out.found.nnz(bad);
    end
    a  = out.truth.sum - out.seed.sum;
    b  = out.found.sum - out.found_vs_seed.sum_min;
    ab = out.found_vs_truth.sum_min - out.found_vs_seed.sum_min;
    xs_rep = 2*ab./(a+b+eps);
  elseif isequal(quantity,'cluster-size')
    xs_rep      = out.found.nnz;
    xs_rep_true = out.truth.nnz;
  elseif isequal(quantity,'cluster-size-th')
    xs_rep      = out.found.nnz_th;
    xs_rep_true = out.truth.nnz_th;
  elseif isequal(quantity,'conductance')
    xs_rep      = 1 - out.found.within ./ (out.found.volume + eps);
    xs_rep_true = 1 - out.truth.within ./ (out.truth.volume + eps);
  elseif isequal(quantity,'conductance-th')
    if ~isfield(out.found,'within_th') out.found.within_th = out.found.within; end
    if ~isfield(out.found,'volume_th') out.found.volume_th = out.found.volume; end
    if ~isfield(out.truth,'within_th') out.truth.within_th = out.truth.within; end
    if ~isfield(out.truth,'volume_th') out.truth.volume_th = out.truth.volume; end
    xs_rep      = 1 - out.found.within_th ./ (out.found.volume_th + eps);
    xs_rep_true = 1 - out.truth.within_th ./ (out.truth.volume_th + eps);
  elseif isequal(quantity,'cputime')
    xs_rep = out.cputime;
  else
    error('don''t know how to calculate %s',quantity);
  end
end

function write_results_table(filename, quantity, seed, variant)
  fprintf('Writing %s\n',filename);
  
  include_truth = 0;
  include_average = 0;
  %[stats,stats_true,datasets,methods] = collect_results(quantity, seed, variant);
  [xs,xs_true,datasets,methods] = collect_results(quantity, seed, variant);
  significance = 0.01;
  
  if isequal(quantity,'f1-scores') || isequal(quantity,'f1-scores-th') || isequal(quantity,'f1-no-seed-scores')
    sign = 1; % higher is better
  elseif isequal(quantity,'conductance') || isequal(quantity,'conductance-th') || isequal(quantity,'cputime')
    sign = -1; % lower is better
  else
    sign = 0; % no bold values
  end
  
  % open
  file = fopen(filename,'wt');
  
  % header
  fprintf(file,'\\begin{tabular}{l');
  if variant.variants_column
    fprintf(file,'l');
  end
  for im = 1:numel(methods)
    if isequal(quantity,'cluster-size') || isequal(quantity,'cluster-size-th') || isequal(quantity,'cputime')
      fprintf(file,'r');
    else
      fprintf(file,'c');
    end
  end
  fprintf(file,'}\n');
  
  fprintf(file,'\\hlinetop\n');
  
  fprintf(file,'Dataset');
  if variant.variants_column
    fprintf(file, ' & Communities');
  end
  if include_truth
    fprintf(file, ' & %s', 'true');
  end
  for im = 1:numel(methods)
    fprintf(file, ' & %s', methods{im}.display_name);
  end
  fprintf(file,' \\\\\n');
  
  fprintf(file,'\\hlinemid\n');
  
  prev_display_name = '';
  
  if include_average
    j = numel(datasets)+1;
    %xs_true{end+1} = cellfun(@mean,xs_true(~cellfun(@isempty,xs_true)));
    xs_true{j} = [];
    xs(j,:) = cell(1,numel(methods));
    for id = 1:numel(datasets)
      if ~isempty(xs_true{id})
        xs_true{j} = [xs_true{j}, mean(xs_true{id})];
      end
      for im = 1:numel(methods)
        if ~isempty(xs{id,im})
          xs{j,im} = [xs{j,im}, mean(xs{id,im})];
        end
      end
    end
    datasets{j} = struct('name','avg','display_name','Average', 'display_variant','');
  end
  
  for id = 1:numel(datasets)
    dataset = datasets{id};
    % print dataset name
    if include_average && id == numel(datasets)
      fprintf(file,'\\hlinemid\n');
    end
    if variant.variants_column
      if id == 1 || ~isequal(dataset.display_name, datasets{id-1}.display_name)
        if id > 1
          fprintf(file,'\\hlinemid\n');
        end
        % number of datasets with the same display_name
        num = length(find(cellfun(@(d)isequal(d.display_name, dataset.display_name), datasets(id:end))));
        fprintf(file,'\\multirow{%d}{*}{%s}\n',num,dataset.display_name);
      end
    else
      fprintf(file,'%s',dataset.display_name);
    end
    
    % print variant name
    if variant.variants_column
      fprintf(file,'& %s',dataset.display_variant);
    end
    
    % find best method
    best_x = sign*-inf;
    for im = 1:numel(methods)
      if ~isempty(xs{id,im}) && sign*mean(xs{id,im}) > sign*mean(best_x)
        best_x = xs{id,im};
      end
    end
    
    % print stats
    for im = (1-include_truth):numel(methods)
      if im == 0
        x = xs_true{id};
        is_best = false;
      else
        x = xs{id,im};
        % 1. bold if not worst than the best
        % 2. bold if not significantly worse than the best
        if isempty(x) || sign == 0 || isequal(dataset.name,'avg')
          is_best = false;
        else
          assert(isequal(size(best_x),size(x)), 'Different size from best: %s %s', datasets{id}.name,methods{im}.name);
          assert(numel(best_x)>1);
          if significance==0
            is_best = ~ (sign*mean(x) < sign*mean(best_x))
          else
            try
              if exist('OCTAVE_VERSION','builtin')
                is_best = t_test_2(sign*x, sign*best_x, '<') >= significance;
              else
                [h,p] = ttest(sign*x, sign*best_x, 'Tail','left');
                is_best = p >= significance || isnan(p);
              end
            catch
              im
              id
              mean_x = mean(x)
              mean_best = mean(best_x)
              var_x = var(x)
              var_best = var(best_x)
              lasterr
              warning('T test failed at %s %s', datasets{id}.name,methods{im}.name);
              is_best = 1;
            end
          end
        end
      end
      fprintf(file,' & ');
      if is_best, fprintf(file,'\\textbf{'); end
      if isempty(x)
        fprintf(file,'-');
      elseif isequal(quantity,'cluster-size') || isequal(quantity,'cluster-size-th')
        fprintf(file,'%.1f', mean(x));
      else
        fprintf(file,'%.3f', mean(x));
      end
      if is_best, fprintf(file,'}'); end
    end
    fprintf(file,'\\\\\n');
  end
  
  fprintf(file,'\\hlinebot\n');
  fprintf(file,'\\end{tabular}\n');
  fclose(file);
end

function write_results_plot(filename, quantity, dataset, seed, values, method_name)
  fprintf('Writing %s     \r',filename);
  file = fopen(filename,'wt');
  fprintf(file,'\\addplot+[] coordinates{');
  for i=1:numel(values)
    method = struct('name', method_name(values(i)));
    [xs] = stats_for_experiment(quantity, method, dataset, seed);
    if length(xs) > 0
      fprintf(file,'(%f,%f) ',values(i),mean(xs));
    end
  end
  fprintf(file,'};\n');
  fclose(file);
end

